/*
  KeePass On-Screen Keyboard Plugin
  Copyright (C) 2005-2009 Dominik Reichl <dominik.reichl@t-online.de>
                     2011 Nikolaus Hammler <nikolaus@hammler.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "StdAfx.h"
#include "KeyboardWindows.h"
#include "KeePassLibCpp/SDK/KpSDK.h"

extern IKpAPI* g_pAPI;

CKeyboardWindows::CKeyboardWindows()
	: m_bCloseOSK(FALSE)
{
}

CKeyboardWindows::~CKeyboardWindows()
{
}

void CKeyboardWindows::OpenKeyboard()
{
	HWND hWndParent = g_pAPI->GetMainWindowHandle();

	HMODULE hLib = LoadLibrary(_T("Kernel32.dll"));
	if(hLib == NULL) return;

	LPWOW64DISABLEREDIRECTION lpDisable = (LPWOW64DISABLEREDIRECTION)
		GetProcAddress(hLib, "Wow64DisableWow64FsRedirection");
	LPWOW64REVERTREDIRECTION lpRevert = (LPWOW64REVERTREDIRECTION)
		GetProcAddress(hLib, "Wow64RevertWow64FsRedirection");

	PVOID pOldValue = NULL;
	if(lpDisable != NULL) lpDisable(&pOldValue);

	ShellExecute(hWndParent, _T("open"), _T("osk.exe"), NULL, NULL, SW_SHOW);

	if(lpRevert != NULL) lpRevert(pOldValue);
	FreeLibrary(hLib);
}

void CKeyboardWindows::ShowKeyboard()
{
	CWnd *wnd = CWnd::FindWindow(_T("OSKMainClass"), 0);
	if(!wnd)
	{
		OpenKeyboard();
		m_bCloseOSK = TRUE;
	}
}

void CKeyboardWindows::HideKeyboard()
{
	if(m_bCloseOSK) {
		CWnd *wnd = CWnd::FindWindow(_T("OSKMainClass"), 0);
		if(wnd) {
			wnd->SendMessage(WM_CLOSE);
			m_bCloseOSK = FALSE;
		}
	}
}

